/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#include <qpe/qpeapplication.h>
#include <qmessagebox.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qkeycode.h>
#include "BoardWidget.h"
#include <unistd.h>

#include "tree.hh"
#include "eval.hh"
#include "board.hh"
#include "gnmm.hh"

#include "images/data.c"

#define ANIMATION_SPEED 8

BoardWidget::BoardWidget(QWidget *parent, const char *name, WFlags f) :
    QWidget(parent, name, f)
{
    m_board.loadFromData(board_data_, sizeof (board_data_));
    m_pieces.loadFromData(pieces_data_, sizeof (pieces_data_));
    m_pixmap = 0;
    m_player_color = WHITE;
    m_level = 2;
    m_animations = 1;
    m_moving = 0;
    m_moving_x = -1;
    m_moving_y = -1;
    m_moving_piece = -1;
    m_state = PlayerSet;
    game_init();
}

BoardWidget::~BoardWidget()
{
    game_exit();
    if (m_pixmap) {
	delete m_pixmap;
    }
}

void BoardWidget::paintEvent(QPaintEvent *event)
{
    QPainter painter_pixmap(m_pixmap);
    QPainter painter_widget(this);
    QColor board_color(102, 92, 21);
    QColor black(0, 0, 0);
    int max_x = width();
    int max_y = height() - 22;
    int offs_x = (max_x - SIZE * TILE_SIZE) / 2;
    int offs_y = (max_y - SIZE * TILE_SIZE) / 2;
    int i;
    int j;
    int k;

    painter_pixmap.fillRect(0, 22, max_x, max_y, board_color);
    for (j = 0; j < SIZE; j++) {
	for (i = 0; i < SIZE; i++) {
	    int tile = BoardMapData[j][i];
	    if (tile) {
		int sx = ((tile - 1) % 4) * TILE_SIZE;
		int sy = ((tile - 1)/ 4) * TILE_SIZE;
		painter_pixmap.drawPixmap(offs_x + i * TILE_SIZE, offs_y + j * TILE_SIZE + 22,
		    m_board, sx, sy, TILE_SIZE, TILE_SIZE);
	    }
	}
    }
    for (k = 0; k < 24; k++) {
	if (board.board[k] != EMPTY) {
	    i = move2coord[k][1];
	    j = move2coord[k][0];
	    int sx = (board.board[k] + 1) / 2 * TILE_SIZE;
	    int sy = 0;
	    painter_pixmap.drawPixmap(offs_x + i * TILE_SIZE, offs_y + j * TILE_SIZE + 22,
		m_pieces, sx, sy, TILE_SIZE, TILE_SIZE);
	}
    }
    if (m_moving) {
	int sx = (m_moving_piece + 1) / 2 * TILE_SIZE;
	int sy = 0;
	painter_pixmap.drawPixmap(m_moving_x, m_moving_y, m_pieces, sx, sy, TILE_SIZE, TILE_SIZE);
    }

    QString s("");
    if (m_moving) {
	s = tr("Moving piece...");
    } else {
	switch (m_state) {
	case None:
	    s = tr("Click to start game");
	    break;
	case ComputerSet:
	case ComputerMove:
	    s = tr("Computer is thinking...");
	    break;
	case ComputerTake:
	    s = tr("Computer takes one piece...");
	    break;
	case PlayerSet:
	    s = tr("Set your %1. piece").arg(board.ply / 2 + 1);
	    break;
	case PlayerMoveStart:
	    s = tr("Select which piece to move");
	    break;
	case PlayerMoveEnd:
	    s = tr("Select where to move the piece");
	    break;
	case PlayerTake:
	    s = tr("Take a %1 piece").arg(m_player_color == WHITE ? tr("black") : tr("white"));
	    break;
	}
    }
    painter_pixmap.fillRect(0, 0, max_x, 22, backgroundColor());
    painter_pixmap.setPen(black);
    painter_pixmap.drawText(7, 0, max_x - 7, 22, AlignLeft | AlignVCenter, s);

    painter_pixmap.flush();
    painter_widget.drawPixmap(0, 0, *m_pixmap);
}

void BoardWidget::resizeEvent(QResizeEvent *event)
{
    QWidget::resizeEvent(event);
    if (!m_pixmap || event->size() != event->oldSize()) {
	if (m_pixmap) {
	    delete m_pixmap;
	}
	m_pixmap = new QPixmap(event->size().width(), event->size().height());
    }
}

void BoardWidget::updatePlayerColor(int player_color)
{
    m_player_color = player_color ? BLACK : WHITE;
    start();
}

void BoardWidget::updateLevel(int level)
{
    m_level = level;
    start();
}

void BoardWidget::updateAnimations(int animations)
{
    m_animations = animations;
}

void BoardWidget::start(void)
{
    m_state = m_player_color == BLACK ? None : PlayerSet;
    game_init();
    repaint(false);
}

void BoardWidget::undo(void)
{
    if (history_pos > 1 && (m_state == PlayerSet || m_state == PlayerMoveStart)) {
	algo.StopComputer();
	history_pos = history_pos - 2;
	if (history_pos > 0) {
	    board = history[history_pos - 1];
	} else {
	    board.Init();
	}
	algo.SetBoard(board);
	m_state = board.ply < 18 ? PlayerSet : PlayerMoveStart;
	current_move.Reset();
	gen_valid_moves();
	repaint(false);
    }
}

void BoardWidget::redo(void)
{
    if (history_pos < (history_length - 1) && (m_state == PlayerSet || m_state == PlayerMoveStart)) {
	algo.StopComputer();
	history_pos = history_pos + 2;
	board = history[history_pos - 1];
	algo.SetBoard(board);
	m_state = board.ply < 18 ? PlayerSet : PlayerMoveStart;
	current_move.Reset();
	gen_valid_moves();
	repaint(false);
    }
}

void BoardWidget::mouseReleaseEvent(QMouseEvent *event)
{
    QWidget::mouseReleaseEvent(event);
    if (m_state == None) {
	start();
	if (m_player_color == BLACK) {
	    repaint(false);
	    computerMove();
	}
	repaint(false);
	return;
    }

    int max_x = width();
    int max_y = height() - 22;
    int offs_x = (max_x - SIZE * TILE_SIZE) / 2;
    int offs_y = (max_y - SIZE * TILE_SIZE) / 2;
    int x = event->x() - offs_x;
    int y = event->y() - offs_y - 22;

    if (x < 0 || y < 0 || x >= max_x || y >= max_y) {
	return;
    }

    int col = x / TILE_SIZE;
    int row = y / TILE_SIZE;
    int pos = coord2move[row][col];

    switch (m_state) {
    case PlayerSet:
	if (validplace[row][col] && board.board[pos] == EMPTY) {
	    current_move.end = pos;
	    if (is_valid_move(current_move)) {
		animateMove(-1, current_move.end, m_player_color);
		makeMove();
		repaint(false);
		if (algo.CurrentHasLost() || n_next_valid == 0) {
		    m_state = None;
		    gameOver();
		} else if (board.ply < 18) {
		    m_state = ComputerSet;
		    computerMove();
		} else {
		    m_state = ComputerMove;
		    computerMove();
		}
	    } else {
		m_state = PlayerTake;
		animateMove(-1, current_move.end, m_player_color);
		board.board[current_move.end] = board.next;
	    }
	}
	break;
    case PlayerMoveStart:
	if (validplace[row][col] && board.board[pos] == board.next) {
	    current_move.start = pos;
	    m_state = PlayerMoveEnd;
	}
	break;
    case PlayerMoveEnd:
	current_move.end = pos;
	if (validplace[row][col] && valid_moving(current_move)) {
	    if (is_valid_move(current_move)) {
		animateMove(current_move.start, current_move.end, m_player_color);
		makeMove();
		repaint(false);
		if (algo.CurrentHasLost() || n_next_valid == 0) {
		    m_state = None;
		    gameOver();
		} else if (board.ply < 18) {
		    m_state = ComputerSet;
		    computerMove();
		} else {
		    m_state = ComputerMove;
		    computerMove();
		}
	    } else {
		m_state = PlayerTake;
		animateMove(current_move.start, current_move.end, m_player_color);
		board.board[current_move.start] = EMPTY;
		board.board[current_move.end] = board.next;
	    }
	} else {
	    m_state = PlayerMoveStart;
	    current_move.Reset();
	}
	break;
    case PlayerTake:
	if (validplace[row][col] && board.board[pos] == -board.next) {
	    if (current_move.start != -1) {
		board.board[current_move.start] = board.next;
	    }
	    board.board[current_move.end] = EMPTY;
	    current_move.take = pos;
	    makeMove();
	    animateMove(pos, -1, m_player_color == WHITE ? BLACK : WHITE);
	    repaint(false);
	    if (algo.CurrentHasLost() || n_next_valid == 0) {
		m_state = None;
		gameOver();
	    } else if (board.ply < 18) {
		m_state = ComputerSet;
		computerMove();
	    } else {
		m_state = ComputerMove;
		computerMove();
	    }
	}
	break;
    default:
	return;
    }
    repaint(false);
}

void BoardWidget::animateMove(int start, int end, int piece)
{
    if (!m_animations || (start == -1 && end == -1)) {
	return;
    }

    int max_x = width();
    int max_y = height() - 22;
    int offs_x = (max_x - SIZE * TILE_SIZE) / 2;
    int offs_y = (max_y - SIZE * TILE_SIZE) / 2;
    int start_piece = EMPTY;
    int start_x;
    int start_y;
    int end_piece = EMPTY;
    int end_x;
    int end_y;

    if (start != -1) {
	start_piece = board.board[start];
	start_x = offs_x + move2coord[start][1] * TILE_SIZE;
	start_y = offs_y + move2coord[start][0] * TILE_SIZE + 22;
	board.board[start] = EMPTY;
    } else {
	start_x = (max_x - TILE_SIZE) / 2;
	start_y = (m_player_color == piece) ? max_y + TILE_SIZE : -TILE_SIZE;
    }
    if (end != -1) {
	end_piece = board.board[end];
	end_x = offs_x + move2coord[end][1] * TILE_SIZE;
	end_y = offs_y + move2coord[end][0] * TILE_SIZE + 22;
	board.board[end] = EMPTY;
    } else {
	end_x = (max_x - TILE_SIZE) / 2;
	end_y = (m_player_color == piece) ? max_y + TILE_SIZE : -TILE_SIZE;
    }
    m_moving = 1;
    m_moving_piece = piece;
    m_moving_x = start_x;
    m_moving_y = start_y;
    while (m_moving_x != end_x || m_moving_y != end_y) {
	repaint(false);
	usleep(40000);
	if (end_x < start_x) {
	    m_moving_x = m_moving_x - ANIMATION_SPEED;
	    if (m_moving_x < end_x) {
		m_moving_x = end_x;
	    }
	} else if (end_x > start_x) {
	    m_moving_x = m_moving_x + ANIMATION_SPEED;
	    if (m_moving_x > end_x) {
		m_moving_x = end_x;
	    }
	}
	if (end_y < start_y) {
	    m_moving_y = m_moving_y - ANIMATION_SPEED;
	    if (m_moving_y < end_y) {
		m_moving_y = end_y;
	    }
	} else if (end_y > start_y) {
	    m_moving_y = m_moving_y + ANIMATION_SPEED;
	    if (m_moving_y > end_y) {
		m_moving_y = end_y;
	    }
	}
    }
    m_moving = 0;
    if (start != -1) {
	board.board[start] = start_piece;
    }
    if (end != -1) {
	board.board[end] = end_piece;
    }
}

void BoardWidget::makeMove()
{
    board.DoMove(current_move);
    algo.DoMove(current_move);
    push(board);
    current_move.Reset();
    gen_valid_moves();
}

void BoardWidget::computerMove(void)
{
    current_move = algo.ComputerMove(level2nodes[m_level]);
    if (!algo.ComputerStopped()) {
	int pos = current_move.take;
	animateMove(current_move.start, current_move.end, m_player_color == WHITE ? BLACK : WHITE);
	makeMove();
	animateMove(pos, -1, m_player_color);
	if (algo.CurrentHasLost() || n_next_valid == 0) {
	    m_state = None;
	    gameOver();
	} else if (board.ply < 18) {
	    m_state = PlayerSet;
	} else {
	    m_state = PlayerMoveStart;
	}
    }
}

void BoardWidget::gameOver(void)
{
    QMessageBox::information(this, tr("Nine Men's Morris"),
	tr("Game over\nYou %1").arg(board.next == m_player_color ? tr("loose") : tr("win")));
}

