/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#include <qpe/qpemenubar.h>
#include <qpe/config.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qpopupmenu.h>
#include "Nmm.h"
#include "BoardWidget.h"

Nmm::Nmm(QWidget *parent, const char *name, WFlags f) :
    QMainWindow(parent, name, f)
{
    /* set window style and caption */
    setCaption(tr("Nine Men's Morris"));

    /* create the play field as the central widget */
    m_board_widget = new BoardWidget(this);
    setCentralWidget(m_board_widget);
    m_board_widget->show();

    /* create the configuration/player_color submenu */
    m_playerColorMenu = new QPopupMenu(0, "player_color");
    m_playerColorMenu->setCheckable(true);
    connect(m_playerColorMenu, SIGNAL(activated(int)), this, SLOT(updatePlayerColor(int)));
    connect(m_playerColorMenu, SIGNAL(activated(int)), m_board_widget, SLOT(updatePlayerColor(int)));
    m_playerColorMenu->insertItem(tr("White"), 0);
    m_playerColorMenu->insertItem(tr("Black"), 1);

    /* create the configuration/level submenu */
    m_levelMenu = new QPopupMenu(0, "level_menu");
    m_levelMenu->setCheckable(true);
    connect(m_levelMenu, SIGNAL(activated(int)), this, SLOT(updateLevel(int)));
    connect(m_levelMenu, SIGNAL(activated(int)), m_board_widget, SLOT(updateLevel(int)));
    m_levelMenu->insertItem(tr("Very easy"), 0);
    m_levelMenu->insertItem(tr("Easy"), 1);
    m_levelMenu->insertItem(tr("Medium"), 2);
    m_levelMenu->insertItem(tr("Hard"), 3);
    m_levelMenu->insertItem(tr("Very hard"), 4);

    /* create the configuration/animations submenu */
    m_animationsMenu = new QPopupMenu(0, "animations_menu");
    m_animationsMenu->setCheckable(true);
    connect(m_animationsMenu, SIGNAL(activated(int)), this, SLOT(updateAnimations(int)));
    connect(m_animationsMenu, SIGNAL(activated(int)), m_board_widget, SLOT(updateAnimations(int)));
    m_animationsMenu->insertItem(tr("No"), 0);
    m_animationsMenu->insertItem(tr("Yes"), 1);
    
    /* create the game submenu */
    QPopupMenu *gameMenu = new QPopupMenu(0, "game_menu");
    gameMenu->insertItem(tr("New"), m_board_widget, SLOT(start()));
    gameMenu->insertSeparator();
    gameMenu->insertItem(tr("Undo"), m_board_widget, SLOT(undo()));
    gameMenu->insertItem(tr("Redo"), m_board_widget, SLOT(redo()));
    gameMenu->insertSeparator();
    gameMenu->insertItem(tr("Close"), this, SLOT(close()));

    /* create the configuration submenu */
    QPopupMenu *confMenu = new QPopupMenu(0, "conf_menu");
    confMenu->insertItem(tr("Player Color"), m_playerColorMenu);
    confMenu->insertItem(tr("Level"), m_levelMenu);
    confMenu->insertItem(tr("Animations"), m_animationsMenu);

    /* create the menu */
    QPEMenuBar *menu = new QPEMenuBar(this);
    menu->insertItem(tr("Game"), gameMenu);
    menu->insertItem(tr("Configuration"), confMenu);

    /* read the configuration file */
    readConfig();

    /* check the menu entries of game mode submenu */
    m_playerColorMenu->setItemChecked(m_player_color, true);
    m_levelMenu->setItemChecked(m_level, true);
    m_animationsMenu->setItemChecked(m_animations, true);
}

Nmm::~Nmm()
{
    writeConfig();
}

void Nmm::readConfig(void)
{
    Config cfg("Nmm");

    cfg.setGroup("Settings");

    int player_color = cfg.readNumEntry("PlayerColor", 0);
    if (player_color <= 0) {
	m_player_color = 0;
    } else {
	m_player_color = 1;
    }
    m_board_widget->updatePlayerColor(m_player_color);

    int level = cfg.readNumEntry("Level", 0);
    if (level <= 0) {
	m_level = 0;
    } else if (level >= 4) {
	m_level = 4;
    } else {
	m_level = level;
    }
    m_board_widget->updateLevel(m_level);

    int animations = cfg.readNumEntry("Animations", 1);
    if (animations <= 0) {
	m_animations = 0;
    } else {
	m_animations = 1;
    }
    m_board_widget->updateAnimations(m_animations);
}

void Nmm::writeConfig(void)
{
    Config cfg("Nmm");

    cfg.setGroup("Settings");
    cfg.writeEntry("PlayerColor", m_player_color);
    cfg.writeEntry("Level", m_level);
    cfg.writeEntry("Animations", m_animations);
}

void Nmm::updatePlayerColor(int player_color)
{
    m_playerColorMenu->setItemChecked(m_player_color, false);
    m_player_color = player_color;
    m_playerColorMenu->setItemChecked(m_player_color, true);
    writeConfig();
}

void Nmm::updateLevel(int level)
{
    m_levelMenu->setItemChecked(m_level, false);
    m_level = level;
    m_levelMenu->setItemChecked(m_level, true);
    writeConfig();
}

void Nmm::updateAnimations(int animations)
{
    m_animationsMenu->setItemChecked(m_animations, false);
    m_animations = animations;
    m_animationsMenu->setItemChecked(m_animations, true);
    writeConfig();
}

